@@ -27,6 +27,10 @@ class TestHierarchicalBreakDown(unittest.TestCase):
     def runTest(self):
         pass
 
+    def test_logp(self):
+        for model in self.models:
+            model.logp
+
     def test_map(self):
         np.random.seed(123)
         for model in self.models:
@@ -69,12 +73,21 @@ class TestHierarchicalBreakDown(unittest.TestCase):
     def test_plot_posteriors(self):
         pass
 
-    @unittest.skip("TODO")
-    def test_subj_by_subj_map_init(self):
-        models = create_test_models()
-        for model in models:
-            if model.is_group_model:
-                model.subj_by_subj_map_init(runs=1)
+    def test_map(self):
+        for model in self.models:
+            model.draw_from_prior()
+            model.approximate_map(minimizer='Powell', use_basin=False, fall_to_simplex=False, individual_subjs=True, minimizer_kwargs={'maxiter': 100})
+            model.draw_from_prior()
+            model.approximate_map(minimizer='Powell', use_basin=True, fall_to_simplex=False, individual_subjs=False, minimizer_kwargs={'maxiter': 100}, basin_kwargs={'T': 2})
+            model.draw_from_prior()
+            model.approximate_map(minimizer='Powell', use_basin=False, fall_to_simplex=True, cycles=2, minimizer_kwargs={'maxiter': 100})
+            model.draw_from_prior()
+            model.approximate_map(minimizer='Powell', use_basin=True, fall_to_simplex=True, minimizer_kwargs={'maxiter': 100})
+            model.draw_from_prior()
+            model.approximate_map(minimizer='Nelder-Mead', use_basin=False, fall_to_simplex=False, minimizer_kwargs={'maxiter': 100})
+            model.draw_from_prior()
+            model.approximate_map(minimizer='Nelder-Mead', use_basin=True, fall_to_simplex=False, minimizer_kwargs={'maxiter': 100})
+            return
 
 
 class TestModelCreation(unittest.TestCase):
@@ -156,7 +169,7 @@ class TestEstimation(unittest.TestCase):
         model = HNodeSimple(data, depends_on={'mu': 'condition'})
 
         model.approximate_map()
-
+        model.approximate_map(individual_subjs=False)
         counter = 0
         for condition, subj_params in params_true.iteritems():
             nodes = model.nodes_db[model.nodes_db['condition'] == condition]
