@@ -1,13 +1,19 @@
 from __future__ import division
 import pickle
+import sys
+import string
+import copy
 
 import numpy as np
+import pandas as pd
 import pymc as pm
-import sys
 
 def flatten(l):
     return reduce(lambda x, y: list(x)+list(y), l)
 
+def pretty_tag(tag):
+    return tag[0] if len(tag) == 1 else string.join(str(tag), ', ')
+
 def load(fname):
     """Load a hierarchical model saved to file via
     model.save(fname)
@@ -94,47 +100,37 @@ def interpolate_trace(x, trace, range=(-1,1), bins=100):
 
     return interp
 
-def save_csv(data, fname, sep=None):
+def save_csv(data, fname, *args, **kwargs):
     """Save record array to fname as csv.
 
     :Arguments:
         data <np.recarray>: Data array to output.
         fname <str>: File name.
 
-    :Optional:
-        sep <str=','>: Separator between columns.
+    :Notes:
+        Forwards call to pandas DataFrame.to_csv
 
     :SeeAlso: load_csv
     """
-    if sep is None:
-        sep = ','
-    with open(fname, 'w') as fd:
-        # Write header
-        fd.write(sep.join(data.dtype.names))
-        fd.write('\n')
-        # Write data
-        for line in data:
-            line_str = [str(i) for i in line]
-            fd.write(sep.join(line_str))
-            fd.write('\n')
+    pd.DataFrame(data).to_csv(fname, *args, **kwargs)
+
 
 def load_csv(*args, **kwargs):
     """Load record array from csv.
 
     :Arguments:
         fname <str>: File name.
-        See numpy.recfromcsv
+        See pandas.read_csv()
 
     :Optional:
-        See numpy.recfromcsv
+        See pandas.read_csv()
 
     :Note:
-        Direct wrapper for numpy.recfromcsv().
+        Forwards call to pandas.read_csv()
 
-    :SeeAlso: save_csv, numpy.recfromcsv
+    :SeeAlso: save_csv, pandas.read_csv()
     """
-    #read data
-    return np.recfromcsv(*args, **kwargs)
+    return pd.read_csv(*args, **kwargs)
 
 
 def set_proposal_sd(mc, tau=.1):
@@ -145,6 +141,32 @@ def set_proposal_sd(mc, tau=.1):
 
     return
 
+def stochastic_from_dist(*args, **kwargs):
+    return pm.stochastic_from_dist(*args, dtype=np.dtype('O'),
+                                   mv=True, **kwargs)
+
+def concat_models(models, concat_traces=True):
+    """Concatenate traces of multiple identical models into a new
+    model containing all traces of the individual models.
+
+    """
+    # copy first model
+    target_model = copy.deepcopy(models[0])
+    target_stochs = target_model.get_stochastics()
+    # append traces
+    for i, model in enumerate(models[1:]):
+        stochs = model.get_stochastics()
+        for node, target_node in zip(stochs.node, target_stochs.node):
+            assert node.__name__ == target_node.__name__, "Node names do not match. You have to pass identical models."
+            if concat_traces:
+                target_node.trace._trace[0] = np.concatenate([target_node.trace[:], node.trace[:]])
+            else:
+                target_node.trace._trace[i+1] = node.trace[:]
+
+    target_model.gen_stats()
+
+    return target_model
+
 
 ###########################################################################
 # The following code is directly copied from Twisted:
